﻿/*
 * Created by SharpDevelop.
 * User: phil
 * Date: 2010.10.13.
 * Time: 14:22
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.IO;

namespace TetheredSun.IO
{
	[Obsolete("Use the WavData class instead.")]
	public class WavWriter: BinaryWriter
	{
		protected uint chunkID;			// 0x52494646
		protected uint chunkSize; 		// size of the RIFF, FMT and DATA chunks - size of ChunkID and Size
		protected uint format;			// 0x57415645
		protected uint subChunk1ID;		// 0x666d7420
		protected uint subChunk1Size; 	// size of the rest of this subchunks from here, 16 for PCM
		protected ushort audioFormat;	// PCM: 1
		protected ushort numberOfChannels;
		protected uint sampleRate;
		protected uint byteRate;		// SampleRate * NumChannels * BitsPerSample / 8
		protected ushort blockAlign; 	// NumChannels * BitsPerSample / 8
		protected ushort bitsPerSample;
		protected uint subChunk2ID;
		protected uint subChunk2Size; 	// NumberOfSamples * NumberOfChannels * BitsPerSample / 8

		protected uint numberOfSamples;
		protected uint currentChannelInfoPosition; 		// position of the data in the file
		protected uint currentMarkerPosition; 			// position of the data in the file
		protected uint currentUnnamedMarkerPosition; 	// position of the data in the file
		protected const int dataPosition = 44;

		
		public WavWriter(Stream stream) : base(stream) 
		{
			chunkID = 0x46464952; // RIFF, little endian
			chunkSize = 0; // size of the RIFF, fmt and data chunks - size of ChunkID and Size
			format = 0x45564157; // WAVE
			subChunk1ID = 0x20746d66; // fmt
			subChunk1Size = 16; // size of the rest of this subchunks from here, 16 for PCM
			audioFormat = 1;	// PCM: 1
			numberOfChannels = 1;
			sampleRate = 1000;
			bitsPerSample = 16;
			subChunk2ID = 0x61746164; // data
			byteRate = (sampleRate * numberOfChannels * bitsPerSample) / 8;
			blockAlign = (ushort)(numberOfChannels * bitsPerSample / 8);
			subChunk2Size=0; // NumberOfSamples * NumberOfChannels * BitsPerSample/8
		
			numberOfSamples = 0;
			currentChannelInfoPosition = 0;
			currentMarkerPosition = 0;
			currentUnnamedMarkerPosition = 0;
		}
		
		
		public ushort NumberOfChannels {
			get { return numberOfChannels; }
		}
		
		public uint SampleRate {
			get { return sampleRate; }
			set { sampleRate = value; }
		}
		
		public ushort BitsPerSample {
			get { return bitsPerSample; }
			set { bitsPerSample = value; }
		}
		
		public uint NumberOfSamples {
			get { return numberOfSamples; }
			set { numberOfSamples = value; }
		}
		
		
		public long SeekInData(int position)
		{
			return this.Seek(dataPosition + position, SeekOrigin.Begin);
		}
		
		protected void SetupHeader()
		{
			byteRate = (sampleRate * numberOfChannels * bitsPerSample) / 8;
			blockAlign = (ushort)(numberOfChannels * bitsPerSample / 8);
			subChunk2Size = numberOfSamples * numberOfChannels * bitsPerSample / 8;
			chunkSize = 4 + (8 + subChunk1Size) + (8 + subChunk2Size);
		}
		
		public void Write(double[] data)
		{
			numberOfSamples = (uint)data.Length;
			WriteHeader();
			SeekInData(0);

			this.Write(GetBytes(data));
		}
		
		protected void WriteHeader()
		{
			SetupHeader();
			this.Seek(0, SeekOrigin.Begin);
			this.Write(chunkID);
			this.Write(chunkSize);
			this.Write(format);
			this.Write(subChunk1ID);
			this.Write(subChunk1Size);
			this.Write(audioFormat);
			this.Write(numberOfChannels);
			this.Write(sampleRate);
			this.Write(byteRate);
			this.Write(blockAlign);
			this.Write(bitsPerSample);
			this.Write(subChunk2ID);
			this.Write(subChunk2Size);
		}
		
		protected byte[] GetBytes(double[] data)
		{
			int bytesPerSample = bitsPerSample / 8;
			byte[] bytes = new byte[data.Length * bytesPerSample];
			byte[] element;
			
			// Normalise data:
			double min = Double.MaxValue;
			double max = Double.MinValue;
			double range;
			
			for (int i = 0; i < data.Length; i++) {
				if (data[i] < min) min = data[i];
				if (data[i] > max) max = data[i];
			}
			
			range = max - min;
			
			if (bytesPerSample == 2) {
				short s;
				int range16 = Int16.MaxValue - Int16.MinValue;
				
				for (int i = 0; i < data.Length; i++) {
					s = (short)(Int16.MinValue + range16 * (data[i] - min) / range);
					element = BitConverter.GetBytes(s);
					// element[0]: lower-order byte.
					// element[1]: higher-order byte.
					bytes[2 * i] = element[0];
					bytes[2 * i + 1] = element[1];
				}
			}
			
			return bytes;
		}
	}
}
