﻿/*
 * Készítette a SharpDevelop.
 * Felhasználó: Ouroboros
 * Dátum: 2010.10.29.
 * Idő: 16:02
 * 
 * A sablon megváltoztatásához használja az Eszközök | Beállítások | Kódolás | Szabvány Fejlécek Szerkesztését.
 */
using System;
using System.ComponentModel;

namespace TetheredSun.Mathematics
{
	/// <summary>
	/// Description of GaussNewtonFitter.
	/// </summary>
	public class GaussNewtonFitter
	{
		protected IFitFunction function;
		protected double error = Double.NaN;
		protected double errorLimit = 0.02;
		protected int maxIterations = 10000000;
		protected int iterations = 0;
		protected Matrix delta;
		protected double[] x;
		protected double[] y;
		protected double[] residuals;
		protected BackgroundWorker worker;
		
		public GaussNewtonFitter(IFitFunction function, double[] x, double[] y)
		{
			this.function = function;
			this.x = x;
			this.y = y;
			worker = new BackgroundWorker();
			worker.DoWork += HandleDoWorkEvents;
			worker.RunWorkerCompleted += HandleRunWorkerCompletedEvents;
			worker.WorkerSupportsCancellation = true;
		}
		
		
		protected double GetSquareSum()
		{
			double sum = 0.0;
			
			for (int i = 0; i < residuals.Length; i++) {
				sum += residuals[i] * residuals[i];
			}
			
			return sum;
		}
		
		protected void HandleDoWorkEvents(object sender, DoWorkEventArgs e)
		{
			while (Iterate());
		}
		
		protected void HandleRunWorkerCompletedEvents(object sender, RunWorkerCompletedEventArgs e)
		{
			if (e.Cancelled) {
				
			} else if (e.Error != null) {
				e.Error.Data.Add("Completed iterations", iterations);
				e.Error.Data.Add("Fit function parameters", function.Parameters);
				e.Error.Data.Add("Current fit error", error);
				throw e.Error;
			} else {
				
			}
		}
		
		protected bool Iterate()
		{
			residuals = function.GetResiduals(x, y);
			Matrix jacobian = function.GetJacobian(x, y);
			Matrix transpose = jacobian.Transpose;
			Matrix left = transpose * jacobian;
			Matrix right = transpose * Matrix.CreateColumnVector(residuals);
			delta = Matrix.Solve(left, right);
			
			for (int i = 0; i < delta.RowCount; i++) {
				function.Parameters[i] += delta[i][0];
			}
			
			iterations++;
			error = GetSquareSum();
			return (error > errorLimit) && (iterations < maxIterations);
		}
		
		public void StartIteration()
		{
			if (worker == null) return;
			iterations = 0;
			error = Double.NaN;
			if (!worker.IsBusy) worker.RunWorkerAsync();
		}
		
		public void StopIteration()
		{
			if (worker == null) return;
			worker.CancelAsync();
		}
	}
}
